/*
 *  dataerr.c  --  Error class with data
 */


#include "dataerr.h"



static void de_dataerr_mark( struct de_dataerr_data *ptr);
static void de_dataerr_free( struct de_dataerr_data *ptr);
extern VALUE de_dataerr_new( void);

static VALUE de_dataerr_ruby_string( VALUE self);
static VALUE de_dataerr_plain_string(  VALUE self);

static VALUE de_obj_raise_it_bang( VALUE obj);
static VALUE de_obj_return_it_bang( VALUE obj);



static VALUE rb_eDataErr;



void
de_dataerr_mark( struct de_dataerr_data *ptr)
{
    rb_gc_mark( ptr->ruby_string);
}

void
de_dataerr_free( struct de_dataerr_data *ptr)
{
    free( ptr->plain_string);
}

VALUE
de_dataerr_new( void)
{
    struct de_dataerr_data *r;
    VALUE de, msg;

    de = Data_Make_Struct( rb_eDataErr, struct de_dataerr_data,
                                &de_dataerr_mark, &de_dataerr_free, r);
    r->plain_string = ALLOC_N( char, 4);
    sprintf( r->plain_string, "foo");
    r->ruby_string = rb_str_new2( "bar");
    msg = rb_str_new2( "An error with a DATA structure.");
    rb_obj_call_init( de, 1, &msg);
    return de;
}



VALUE
de_dataerr_ruby_string( VALUE self)
{
    struct de_dataerr_data *r;

    Data_Get_Struct( self, struct de_dataerr_data, r);
    return r->ruby_string;
}

VALUE
de_dataerr_plain_string( VALUE self)
{
    struct de_dataerr_data *r;

    Data_Get_Struct( self, struct de_dataerr_data, r);
    return rb_str_new2( r->plain_string);
}




VALUE
de_obj_raise_it_bang( VALUE obj)
{
    rb_exc_raise( de_dataerr_new());
    return Qnil;
}

VALUE
de_obj_return_it_bang( VALUE obj)
{
    return de_dataerr_new();
}




void
Init_dataerr( void)
{
    rb_eDataErr = rb_define_class( "DataError", rb_eStandardError);
    rb_undef_method( CLASS_OF( rb_eDataErr), "new");

    rb_define_method( rb_eDataErr, "ruby_string", &de_dataerr_ruby_string, 0);
    rb_define_method( rb_eDataErr, "plain_string", &de_dataerr_plain_string, 0);

    rb_define_method( rb_cObject, "raise_it!", &de_obj_raise_it_bang, 0);
    rb_define_method( rb_cObject, "return_it!", &de_obj_return_it_bang, 0);
}

